#include <visa.h>

/*= Useful Macros ===========================================================*/

/* - Basic Macros ---------------------------------------------------------- */

/* Error checking macro */
#define CHECKERR(fCal) if (gmpsp_status = (fCal), gmpsp_status < VI_SUCCESS) \
    return gmpsp_status

/* Error checking macro with jump to the ERROR: label */
#define GOCHECKERR(fCal) if (gmpsp_status = (fCal), gmpsp_status < VI_SUCCESS) \
                            goto ERROR_LABEL
/**#define GOCHECKERR(fCal) if (gmpsp_status = (fCal), gmpsp_status < VI_SUCCESS) \
                            goto ERROR_LABEL; else
*/
/* Parameter checking macro */
#define CHECKPARAM(fCal,param) if (fCal != VI_SUCCESS) return VI_ERROR_PARAMETER##param

/* - Additional Macros ----------------------------------------------------- */

/* Remap parameter error code */
#define REMAP(par_in, par_out) if (gmpsp_status == VI_ERROR_PARAMETER##par_in)\
                                    gmpsp_status = VI_ERROR_PARAMETER##par_out;\
                                if (gmpsp_status < VI_SUCCESS)\
                                    return gmpsp_status

/* Check NULL pointer */
#define CHECKNULL(pointer, param) if (pointer == NULL) return VI_ERROR_PARAMETER##param


/*****************************************************************************/
/*= INSTRUMENT-DEPENDENT STATUS/RANGE STRUCTURE  ============================*/
/*****************************************************************************/
typedef struct  gmpsp_stringValPair
{
   ViStatus stringVal;
   ViString stringName;
}  gmpsp_tStringValPair;

struct gmpsp_statusDataRanges
{
    ViChar              instrDriverRevision[256];
    ViBoolean           errorChecking;
};

typedef struct gmpsp_statusDataRanges *gmpsp_instrPtr;

ViStatus _VI_FUNC gmpsp_delay (/*@unused@*/ViSession instrumentHandle, ViReal64 seconds);
ViBoolean gmpsp_invalidViBooleanRange (ViBoolean val);
ViBoolean gmpsp_invalidViInt32Range (ViInt32 val, ViInt32 min, ViInt32 max);
ViBoolean gmpsp_invalidViReal64Range (ViReal64 val, ViReal64 min,
                                       ViReal64 max);
ViStatus gmpsp_checkStatus (ViSession instrSession);
ViStatus gmpsp_waitOPC (ViSession instrSession, ViPString command,
                        ViPString instrResp, ViInt32 timeout);
ViStatus gmpsp_sendCmd (ViSession instrSession, ViString cmd);
ViStatus gmpsp_setViInt32 (ViSession instrSession,
                           ViString cmd,
                           ViInt32 value,
                           ViInt32 min,
                           ViInt32 max,
                           ViStatus parameterErrorCode);
ViStatus gmpsp_getViInt32 (ViSession instrSession,
                           ViString cmd,
                           ViInt32 *value,
                           ViStatus parameterErrorCode);
ViStatus gmpsp_setViBoolean (ViSession instrSession,
                           ViString cmd,
                           ViBoolean value,
                           ViStatus parameterErrorCode);
ViStatus gmpsp_getViBoolean (ViSession instrSession,
                           ViString cmd,
                           ViBoolean *value,
                           ViStatus parameterErrorCode);
ViStatus gmpsp_setViReal64 (ViSession vi,
                            ViString cmd,
                            ViReal64 value,
                            ViReal64 min,
                            ViReal64 max,
                            ViStatus parameterErrorCode);
ViStatus gmpsp_getViReal64 (ViSession instrSession,
                            ViString cmd,
                            ViReal64 *value,
                            ViStatus parameterErrorCode);
ViStatus gmpsp_setViString (ViSession instrSession,
                            ViString cmd,
                            ViString value,
                            ViStatus parameterErrorValue);
ViStatus gmpsp_getViString (ViSession instrSession,
                            ViString cmd,
                            ViString value,
                            ViUInt32 buffer_size,
                            ViStatus parameterErrorCode);
ViStatus gmpsp_setEnum (ViSession instrSession,
                        ViString cmd,
                        ViInt32 value,
                        ViString list[],
                        ViStatus parameterErrorCode);
ViStatus gmpsp_getEnum (ViSession instrSession,
                        ViString cmd,
                        ViInt32 *value,
                        ViString list[],
                        ViStatus parameterErrorCode);
ViStatus gmpsp_checkIndexInList (ViInt32 index,
                                 ViString list[],
                                 ViStatus statusIfError);
ViStatus gmpsp_getIndexFromString (ViString string,
                                   ViString *list,
                                   ViInt32 *index);
ViStatus gmpsp_readDataUnknownLength(ViSession instrSession,
                                     /*@null@*/ViChar **outBuffer,
                                     ViUInt32 *count);
ViStatus gmpsp_getViUInt32 (ViSession instrSession,
                           ViString cmd,
                           ViUInt32 *value,
                           ViStatus parameterErrorCode);
